;- Beispiel fr RSQRTSS
;- Reciprocal of Sqare Root of Scalar Single-FP Value
;- SSE - zieht die Quadrat-Wurzel aus der (untersten) Fliesskommazahl einfacher Genauigkeit
;-  des 2.Operanden, bildet den Kehrwert und speichert das Ergebnis im 1.Operanden
;- Dateiname 3213.pb
;- "Helle" Klaus Helbing, 08.04.2007, PB v4.02
;- Operand2 kann ein XMM-Register oder eine 32-Bit-Variable sein, Operand1 muss ein XMM-Register sein
;- Die Rundungskontrolle des MXCSR-Registers spielt keine Rolle
;- Der relative Fehler liegt bei ca. <= 1.5*2^-12 (lt.Intel)

Global A.f = 1234.5

!rsqrtss xmm0,[v_A]
!movups dqword[R],xmm0

MessageRequester("RSQRTSS   Reziproker Wert der Quadrat-Wurzel von A", StrF(PeekF(?R)))

;------------------------------------------------------------------------------
;- Anwendungs-Beispiel Berechnung Quadrat-Wurzel 
Global Radikand.f = 50000000.0
Global SqrPB.f
Global SqrSSE.f

;- Bentigt CPU mit SSE-Untersttzung --------------------- 
Macro SqrSSE(r,e)
     LEA edi,r               ;Speicher-Adresse von Radikand 
     !movss xmm0,[edi] 
     !rsqrtss xmm1,xmm0
     !rcpss xmm0,xmm1 
     LEA edi,e               ;Speicher-Adresse von Ergebnis 
     !movss [edi],xmm0 
EndMacro

;- Test SSE
T1= ElapsedMilliseconds() 
For i = 0 To 9999999 
  SqrSSE(Radikand, SqrSSE)                 ;Variablen freier Wahl
Next 
T1 = ElapsedMilliseconds() - T1 

;- Test PB
T2= ElapsedMilliseconds() 
For i = 0 To 9999999 
  SqrPB = Sqr(Radikand) 
Next 
T2 = ElapsedMilliseconds() - T2 

Sqr$ = "Test fr Radikand = " + StrF(Radikand) + #CRLF$ + #CRLF$
Sqr$ + "SSE : " + StrF(SqrSSE) + " in " + Str(T1) + " ms" + #CRLF$ 
Sqr$ + "PB   : " + StrF(SqrPB) + " in " + Str(T2) + " ms" + #CRLF$ 
MessageRequester("Quadrat-Wurzel mittels RSQRTSS und RCPSS", Sqr$) 

End

!section '.data' Data readable writeable
R:
!R DQ ?                                ;1 Quad-Word reservieren (direkt ein Double-Quad-Word geht nicht)
!times 1 DQ ?                          ;noch ein weiteres Quad-Word reservieren
; IDE Options = PureBasic v4.02 (Windows - x86)
; CursorPosition = 28
; FirstLine = 12
; Folding = -
; EnableAsm
; DisableDebugger