;- bersicht mit Beispielen fr die SIMD-Befehle (MMX, 3DNow!, Extended 3DNow!, SSE, SSE2, SSE3,
;-  SSSE3, SSE4.1 und SSE4.2). Befehle fr 64-Bit-Betriebssysteme wurden nicht bercksichtigt.
;- Die beiden "Nur 64-Bit-Betriebssystem-Befehle" sind: PEXTRQ und PINSRQ (beide SSE4.1)
;- "Helle" Klaus Helbing, 22.02.2008, PB v4.10
;- Zum Ausfhren der Beispiele unbedingt mehr als nur eine Instanz zulassen (Datei/Einstellungen)! 
;- Die Register-Bezeichnungen in den Beispielen (z.B. MM0, XMM5 usw.) sind willkrlich gewhlt
;-  und entsprechend nderbar
;- Die Beispiele erheben keinen Anspruch auf Originalitt oder Optimalitt ;-) ! 
;- Allgemeine Syntax fr die Befehle in den Beispielen (Anzahl der Operanden je nach Befehl):
;-  Befehl Operand1, Operand2, Operand3
;- Insbesonders bei Floating-Point-Move- und Logischen Verknpfungs-Befehlen wurden wegen besserer
;-  Veranschaulichung auch mal Integer-(Hex-) anstatt FP-Variable verwendet (was durchaus auch in
;-  der Praxis mglich ist)  
;- Im PureBasic\Compilers-Verzeichnis ist die FAsm.exe auf den neuesten Stand zu bringen! 
;- Die Verfgbarkeit der jeweiligen SIMD-Erweiterung (CPU-abhngig) wird durch ein nachgestelltes "+"
;-  bzw. "--" angezeigt
;- Wenn nicht zur Beschreibung des Befehls anders notwendig werden Unalignment-Move-Befehle
;-  verwendet (langsamer, aber erstmal sicherer) 
;- Kann fr 128-Bit-Variablen kein 16-Byte-Alignment sichergestellt werden knnen, ist der Umweg
;-  ber einen Unalignment-XMM-Register-Move-Befehl zu machen 
;- Mitunter aus Platzmangel verwendete Abkrzungen:
;-      FP          - Floating-Point, Fliesskomma-Zahl
;-      DP          - Double-Precision, 64-Bit Floating-Point
;-      SP          - Single-Precision, 32-Bit Floating-Point
;-      Op.         - Operand
;-      D-Word      - Double-Word, 32-Bit 
;-      D-Quad      - Double-Quad, 128-Bit 
;- Anzahl aufgefhrter Befehle (22.02.2008):
;  MMX ...............  49
;  3DNow! ............  23
;  Extended 3DNow! ...  24
;  SSE ...............  65
;  SSE2 ..............  69
;  SSE3 ..............  13
;  SSSE3 .............  16
;  SSE4.1 ............  47   ohne die beiden reinen 64-Bit-Betriebssystem-Befehle
;  SSE4.2 ............   7                      
;                      ===
;                      313

;---------------------------------------------------------------------------------------------------

Global DateiOld = 20000                ;fr Datei-Auswahl
Global TMMX$ = "    MMX   " + Chr(151) ;sieht besser aus als "--"
Global T3DNow$ = "   3DNow!   " + Chr(151)
Global Te3DNow$ = "   Extended 3DNow!   " + Chr(151)
Global TSSE$ = "   SSE   " + Chr(151)
Global TSSE2$ = "    SSE2   " + Chr(151)
Global TSSE3$ = "    SSE3   " + Chr(151)
Global TSSSE3$ = "    SSSE3   " + Chr(151)
Global TSSE41$ = "    SSE4.1  " + Chr(151)
Global TSSE42$ = "    SSE4.2  " + Chr(151)
Global TPOPCNT$ = "    SSE4.2 - POPCNT  " + Chr(151)

;- Ermittlung, welche SIMD-Erweiterungen die vorhandene CPU untersttzt
Global Bit0.l  = $1          ;fr SSE3
Global Bit9.l  = $200        ;fr SSSE3
Global Bit19.l = $80000      ;fr SSE4.1
Global Bit20.l = $100000     ;fr SSE4.2
Global Bit21.l = $200000     ;fr EFlag
Global Bit23.l = $800000     ;fr MMX und POPCNT
Global Bit25.l = $2000000    ;fr SSE
Global Bit26.l = $4000000    ;fr SSE2
Global Bit30.l = $40000000   ;fr extended 3DNow!
Global Bit31.l = $80000000   ;fr 3DNow!

;-------- Test, ob der CPUID-Befehl berhaupt vom Prozessor verarbeitet werden kann
;-------- ist mglich, wenn Bit21 des EFlag-Registers verndert werden kann
    !pushfd                  ;das EFlag-Register (32-Bit) auf den Stack
    !pop eax                 ;rein in EAX
    !mov edx,eax             ;EAX unverndert lassen
    !xor edx,[v_Bit21]       ;Bit21 kippen 
    !push edx                          
    !popfd                   ;in EFlag schreiben
    !pushfd                  ;wieder auf Stack
    !pop edx
    !push eax                ;der Ordnung halber alten Wert wieder herstellen
    !popfd   
    !cmp eax,edx
    !je l_noext              ;sind gleich -> CPUID ist nicht mglich

;-------- Test auf MMX, SSE, SSE2, SSE3 und SSSE3
;-------- vorhanden, wenn im Rckgaberegister EDX bzw. ECX die entsprechenden Bits gesetzt sind
    !mov eax,1h
    !cpuid
    !test edx,[v_Bit23]      ;MMX
    !jz l_obsse              ;falls Tests auf "ltere" SIMD-Befehle mal wegfallen wird stur weiter getestet                
    !push ecx  
    !push edx
  TMMX$ = "    MMX   +"
    !pop edx
    !pop ecx
OBSSE:
    !test edx,[v_Bit25]      ;SSE  
    !jz l_obsse2
    !push ecx  
    !push edx  
  TSSE$ = "   SSE   +"
    !pop edx
    !pop ecx
OBSSE2:
    !test edx,[v_Bit26]      ;SSE2  
    !jz l_obsse3
    !push ecx  
  TSSE2$ = "    SSE2   +"
    !pop ecx
OBSSE3:
    !test ecx,[v_Bit0]       ;SSE3
    !jz l_obssse3
    !push ecx  
  TSSE3$ = "    SSE3   +"
    !pop ecx  
OBSSSE3:
    !test ecx,[v_Bit9]       ;SSSE3
    !jz l_obsse41
    !push ecx  
  TSSSE3$ = "    SSSE3   +"
    !pop ecx
OBSSE41: 
    !test ecx,[v_Bit19]      ;SSE4.1
    !jz l_obsse42
    !push ecx  
  TSSE41$ = "    SSE4.1   +"
    !pop ecx  
OBSSE42:
    !test ecx,[v_Bit20]      ;SSE4.2
    !jz l_obpopcnt
    !push ecx
  TSSE42$ = "    SSE4.2   +"
    !pop ecx
OBPOPCNT:
    !test ecx,[v_Bit23]      ;POPCNT
    !jz l_testende
  TPOPCNT$ = "    SSE4.2 - POPCNT   +"
TestEnde:

;-------- Anzahl der vorhandenen extended levels ermitteln als Vorstufe fr 3DNow!-Test
;-------- Rckgabewert in EAX (-80000000h) gibt Anzahl der extended level an
    !mov eax,80000000h
    !cpuid
    !cmp eax,80000000h       ;hat nichts mit einem Bit zu tun!
    !jbe l_noext             ;keine extended levels, 3DNow! berspringen
;------------------------------------------------------------------------------
    
;-------- Test auf 3DNow! und extended 3DNow!
;-------- vorhanden, wenn im Rckgaberegister EDX das Bit31 bzw. Bit30 gesetzt ist
;-------- 3DNow! steht nur auf AMD-Prozessoren ab dem K6-2 zur Verfgung 
    !mov eax,80000001h       ;80000001h ist praktisch das erste extended level, dieser wird von Intel-Prozessoren nicht untersttzt!
    !cpuid                   ;Intel-Prozessoren liefern hier EAX=0 zurck
    !or eax,eax
    !je l_noext              ;ist Intel-Prozessor
    !test edx,[v_Bit31]      ;3DNow! 
    !jz l_noext
    !push edx
  T3DNow$ = "   3DNow!   +"   
    !pop edx
    !test edx,[v_Bit30]      ;extended 3DNow!  
    !jz l_noext
  Te3DNow$ = "   Extended 3DNow!   +"
NOEXT:    
;------------------------------------------------------------------------------

ExamineDesktops()
  If DesktopWidth(0) < 1024
    MessageRequester("Hinweis", "Eine Bildschirmbreite von mindestens 1024 Pixel ist sehr zu empfehlen!")
  EndIf

  If OpenWindow(0, 0, 0, 950, 400, "SIMD - Befehle", #PB_Window_SystemMenu | #PB_Window_ScreenCentered) And CreateGadgetList(WindowID(0))
   
    ListViewGadget(20, 10, 100, 930, 290)
    
    PanelGadget(0, 8, 8, 934, 60)
      AddGadgetItem(0, -1, TMMX$)
        PanelGadget(1, 5, 5, 920, 30)
          AddGadgetItem(1, -1, "      Register-Befehle      ")
          AddGadgetItem(1, -1, "      Mathematische Befehle      ")
          AddGadgetItem(1, -1, "      Schiebe-Befehle      ")
          AddGadgetItem(1, -1, "      Logische Verknpfungen      ")
          AddGadgetItem(1, -1, "       Vergleiche       ")
          AddGadgetItem(1, -1, "      (Ent)pack-Befehle      ")
          AddGadgetItem(1, -1, "       Sonstiges       ")
      AddGadgetItem(0, -1, T3DNow$)
        PanelGadget(2, 5, 5, 920, 30)  
          AddGadgetItem(2, -1, "           Register-Befehle         ")            
          AddGadgetItem(2, -1, "        Mathematische Befehle       ")
          AddGadgetItem(2, -1, "            Konvertierungen           ")
          AddGadgetItem(2, -1, "               Vergleiche              ")
          AddGadgetItem(2, -1, "            Cache-Befehle           ")
          AddGadgetItem(2, -1, "            Sonstiges               ")
      AddGadgetItem(0, -1, Te3DNow$)
        PanelGadget(3, 5, 5, 920, 30)  
          AddGadgetItem(3, -1, "          Register-Befehle         ")          
          AddGadgetItem(3, -1, "         Mathematische Befehle       ")
          AddGadgetItem(3, -1, "            Konvertierungen            ")
          AddGadgetItem(3, -1, "           Vergleiche          ")          
          AddGadgetItem(3, -1, "            Shuffle-Befehle            ")         
          AddGadgetItem(3, -1, "             Cache-Befehle           ")
      AddGadgetItem(0, -1, TSSE$)
        PanelGadget(4, 5, 5, 920, 30)
          AddGadgetItem(4, -1, "   Register-Befehle  ")
          AddGadgetItem(4, -1, "   Konvertierungen  ")
          AddGadgetItem(4, -1, "   Mathematische Befehle  ")
          AddGadgetItem(4, -1, "    Logische Verknpfungen   ")
          AddGadgetItem(4, -1, "      Vergleiche     ")
          AddGadgetItem(4, -1, "    Shuffle-Befehle   ")
          AddGadgetItem(4, -1, "    Entpack-Befehle   ")      
          AddGadgetItem(4, -1, "    Cache-Befehle   ")      
      AddGadgetItem(0, -1, TSSE2$)
        PanelGadget(5, 5, 5, 920, 30)
          AddGadgetItem(5, -1, " Register-Befehle")
          AddGadgetItem(5, -1, " Konvertierungen")
          AddGadgetItem(5, -1, " Mathematische Befehle")
          AddGadgetItem(5, -1, " Schiebe-Befehle")
          AddGadgetItem(5, -1, " Logische Verknpfungen ")
          AddGadgetItem(5, -1, " Vergleiche ")
          AddGadgetItem(5, -1, " Shuffle-Befehle ")
          AddGadgetItem(5, -1, " Entpack-Bef. ")
          AddGadgetItem(5, -1, " Cache-Bef. ")                            
          AddGadgetItem(5, -1, " Sonstiges ")        
      AddGadgetItem(0, -1, TSSE3$)
        PanelGadget(6, 5, 5, 920, 30)
          AddGadgetItem(6, -1, "                     Register-Befehle                     ")
          AddGadgetItem(6, -1, "                       Konvertierungen                      ")
          AddGadgetItem(6, -1, "                     Mathematische Befehle                     ")
          AddGadgetItem(6, -1, "                            Sonstiges                            ")
      AddGadgetItem(0, -1, TSSSE3$)
        PanelGadget(7, 5, 5, 920, 30)
          AddGadgetItem(7, -1, "                                 Mathematische Befehle                                ")
          AddGadgetItem(7, -1, "                                     Shuffle-Befehle                                    ")          
          AddGadgetItem(7, -1, "                                        Sonstiges                                       ") 
      AddGadgetItem(0, -1, TSSE41$)
        PanelGadget(8, 5, 5, 920, 30)
          AddGadgetItem(8, -1, "               Register-Befehle              ")
          AddGadgetItem(8, -1, "                Konvertierungen              ")
          AddGadgetItem(8, -1, "              Mathematische Befehle              ")
          AddGadgetItem(8, -1, "                    Vergleiche                 ")          
          AddGadgetItem(8, -1, "                 Pack-Befehle                 ")          
      AddGadgetItem(0, -1, TSSE42$)
        PanelGadget(9, 5, 5, 920, 30)
          AddGadgetItem(9, -1, "                                     String-Befehle                                     ")           
          AddGadgetItem(9, -1, "                                           Vergleiche                                           ")         
          AddGadgetItem(9, -1, "                                        Sonstiges                                       ") 
      AddGadgetItem(0, -1, TPOPCNT$)

;--------------------------------------------------------------------------------------------------
    Repeat 
      
         Panel0 = GetGadgetState(0)
;--------------------------------------------------------------------------------------------------
;- Test auf MMX         
       If Panel0 = 0
         Panel1 = GetGadgetState(1)     
           If Init0 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel1Old = 1
             Init0 = 1
             Panel1 = 100              ;irgendein hoher Wert 
           EndIf 
         If Panel1 <> Panel1Old
           Select Panel1
;--- Test auf Register-Befehle             
            Case 0
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "MOVD           kopiert ein Double-Word aus dem zweiten Operanden in den ersten")
             AddGadgetItem(20, -1, "MOVQ           kopiert ein Quad-Word aus den zweiten Operanden in den ersten")
;--- Test auf mathematische Befehle       
            Case 1
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PADDB                addiert die korrespondierenden Bytes der beiden Operanden in den ersten Operanden, berlufe finden nur innerhalb der Bytes statt, werden aber nicht signalisiert")
             AddGadgetItem(20, -1, "PADDW              addiert die korrespondierenden Words der beiden Operanden in den ersten Operanden, berlufe finden nur innerhalb der Words statt, werden aber nicht signalisiert")
             AddGadgetItem(20, -1, "PADDD               addiert die korrespondierenden D-Words der beiden Operanden in den ersten Operanden, berlufe finden nur innerhalb der D-Words statt, werden aber nicht signalisiert")          
             AddGadgetItem(20, -1, "PADDSB             wie PADDB, aber die Bytes werden als vorzeichenbehaftet interpretiert und bei ber- oder Unterlauf wird saturiert (max = 127 (7fh) bzw. min = -128 (80h))")          
             AddGadgetItem(20, -1, "PADDSW            wie PADDW, aber die Words werden als vorzeichenbehaftet interpretiert und bei ber- oder Unterlauf wird saturiert (max = 32767 (7fffh) bzw. min = -32768 (8000h))")          
             AddGadgetItem(20, -1, "PADDUSB          wie PADDB, aber bei berlauf wird saturiert (max = 255 (ffh)")          
             AddGadgetItem(20, -1, "PADDUSW         wie PADDW, aber bei berlauf wird saturiert (max = 65535 (ffffh)")          
             AddGadgetItem(20, -1, "PMADDWD         multipliziert die korrespondierenden Words der beiden Operanden, addiert dann die Double-Words der Ergebnisse und speichert diese im ersten Operanden")          
             AddGadgetItem(20, -1, "PMULHW           multipliziert die korrespondierenden Words der beiden Operanden und speichert die oberen 16 Bit des Ergebnisses im ersten Operanden")          
             AddGadgetItem(20, -1, "PMULLW            multipliziert die korrespondierenden Words der beiden Operanden und speichert die unteren 16 Bit des Ergebnisses im ersten Operanden")          
             AddGadgetItem(20, -1, "PSUBB               subtrahiert die korrespondierenden Bytes der beiden Operanden in den ersten Operanden, berlufe finden nur innerhalb der Bytes statt, werden aber nicht signalisiert")          
             AddGadgetItem(20, -1, "PSUBW              subtrahiert die korrespondierenden Words der beiden Operanden in den ersten Operanden, berlufe finden nur innerhalb der Words statt, werden aber nicht signalisiert")        
             AddGadgetItem(20, -1, "PSUBD               subtrahiert die korrespondierenden D-Words der beiden Operanden in den ersten Operanden, berlufe finden nur innerhalb der D-Words statt, werden aber nicht signalisiert")        
             AddGadgetItem(20, -1, "PSUBSB             wie PSUBB, aber die Bytes werden als vorzeichenbehaftet interpretiert und bei ber- oder Unterlauf wird saturiert (max = 127 (7fh) bzw. min = -128 (80h))")
             AddGadgetItem(20, -1, "PSUBSW            wie PSUBW, aber die Words werden als vorzeichenbehaftet interpretiert und bei ber- oder Unterlauf wird saturiert (max = 32767 (7fffh) bzw. min = -32768 (8000h))")          
             AddGadgetItem(20, -1, "PSUBUSB          wie PSUBB, aber bei Unterlauf wird saturiert (min = 00)")
             AddGadgetItem(20, -1, "PSUBUSW         wie PSUBW, aber bei Unterlauf wird saturiert (min = 0000)")
;--- Test auf Schiebe-Befehle
            Case 2 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PSLLW       shiftet die Bits innerhalb der Words des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach links. Von rechts wird mit Nullen aufgefllt")
             AddGadgetItem(20, -1, "PSLLD        shiftet die Bits innerhalb der Double-Words des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach links. Von rechts wird mit Nullen aufgefllt")
             AddGadgetItem(20, -1, "PSLLQ        shiftet die Bits innerhalb des/der Quad-Words des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach links. Von rechts wird mit Nullen aufgefllt")
             AddGadgetItem(20, -1, "PSRAW      shiftet die Bits innerhalb der Words des 1. Operanden um die im 2. Operanden angegebene Anzahl nach rechts. Von links wird mit Nullen aufgefllt, das MSB bleibt das Vorzeichen-Bit!")           
             AddGadgetItem(20, -1, "PSRAD       shiftet die Bits innerhalb der D-Words des 1.Operanden um die im 2.Operanden angegebene Anzahl nach rechts. Von links wird mit Nullen aufgefllt, das MSB bleibt das Vorzeichen-Bit!")
             AddGadgetItem(20, -1, "PSRLW      shiftet die Bits innerhalb der Words des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach rechts. Von links wird mit Nullen aufgefllt")           
             AddGadgetItem(20, -1, "PSRLD       shiftet die Bits innerhalb der Double-Words des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach rechts. Von links wird mit Nullen aufgefllt")
             AddGadgetItem(20, -1, "PSRLQ       shiftet die Bits innerhalb des/der Quad-Words des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach rechts. Von links wird mit Nullen aufgefllt")           
;--- Test auf logische Verknpfungen
            Case 3
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PAND                 AND-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")
             AddGadgetItem(20, -1, "PANDN              der erste Operand wird NOT-verknpft (invertiert), danach erfolgt eine AND-Verknpfung mit dem zweiten Operanden und das Ergebnis wird im ersten Operanden gespeichert")
             AddGadgetItem(20, -1, "POR                   OR-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")
             AddGadgetItem(20, -1, "PXOR                 XOR-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")           
;--- Test auf Vergleiche 
            Case 4
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PCMPEQB         vergleicht die korrespond. Bytes der beiden Operanden und setzt bei Gleichheit das entsprechende Byte des ersten Operanden auf 0ffh (alle Bits gesetzt), sonst auf Null")
             AddGadgetItem(20, -1, "PCMPEQW        vergleicht die korrespond. Words der beiden Operanden und setzt bei Gleichheit das entsprechende Word des ersten Operanden auf 0ffffh (alle Bits gesetzt), sonst auf Null")
             AddGadgetItem(20, -1, "PCMPEQD        vergleicht die korrespond. D-Words der beiden Operanden und setzt bei Gleichheit das entsprechende D-Word des ersten Operanden auf 0ffffffffh (alle Bits gesetzt), sonst auf Null")
             AddGadgetItem(20, -1, "PCMPGTB         vergleicht die korrespond. Bytes von Operand1 und Operand2. Ist Operand1 grsser, wird das entsprechende Byte des ersten Operanden auf 0ffh gesetzt, sonst auf Null")           
             AddGadgetItem(20, -1, "PCMPGTW        vergleicht die korrespond. Words von Operand1 und Operand2. Ist Operand1 grsser, wird das entsprechende Word des ersten Operanden auf 0ffffh gesetzt, sonst auf Null")
             AddGadgetItem(20, -1, "PCMPGTD        vergleicht die korrespond. D-Words von Operand1 und Operand2. Ist Operand1 grsser, wird das entsprechende D-Word des ersten Operanden auf 0ffffffffh gesetzt, sonst auf Null")           
;--- Test auf (Ent)pack-Befehle
            Case 5
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PACKSSWB        konvertiert die Words in vorzeichenbehaftete Bytes. Die von Operand2 werden in den oberen Bytes von Operand1, die von Op.1 in den unteren Bytes von Operand1 gespeichert")
             AddGadgetItem(20, -1, "PACKSSDW       konvertiert die D-Words in vorzeichenbehaftete Words. Die von Operand2 werden in den oberen Words von Op.1, die von Op.1 in den unteren Words von Operand1 gespeichert")
             AddGadgetItem(20, -1, "PACKUSWB        konvertiert die Words in vorzeichenlose Bytes. Die von Operand2 werden in den oberen Bytes von Operand1, die von Op.1 in den unteren Bytes von Operand1 gespeichert")
             AddGadgetItem(20, -1, "PUNPCKHBW     kopiert das oberste Byte von Op.2 in das oberste Byte von Op.1, das oberste Byte von Op.1 an die 2.Stelle von Op.1, das zweithchste Byte von Op.2 an die nchste Stelle usw.")          
             AddGadgetItem(20, -1, "PUNPCKHWD     kopiert das oberste Word von Op.2 in das oberste Word von Op.1, das oberste Word von Op.1 an die 2.Stelle von Op.1, das zweith. Word von Op.2 an die nchste Stelle usw.")
             AddGadgetItem(20, -1, "PUNPCKHDQ     kopiert das oberste D-Word von Operand2 in das oberste D-Word von Operand1 und das oberste D-Word von Operand1 an die 2.Stelle von Operand1")          
             AddGadgetItem(20, -1, "PUNPCKLBW     wie PUNPCKHBW, nur werden jetzt die Bytes vom zweiten (unteren) Double-Word genommen")
             AddGadgetItem(20, -1, "PUNPCKLWD     wie PUNPCKHWD, nur werden jetzt die Words vom zweiten (unteren) Double-Word genommen")          
             AddGadgetItem(20, -1, "PUNPCKLDQ      wie PUNPCKHDQ, nur wird jetzt das untere Double-Word genommen")
;--- Test auf Sonstiges 
            Case 6
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "EMMS            setzt die Tags der FPU auf 1 und erklrt damit die FPU wieder fr verfgbar. Alle anderen MMX-Befehle setzen die Tags auf Null ! Sollte immer am Ende eines MMX-Blocks stehen")
             AddGadgetItem(20, -1, "FXRSTOR      stellt die mit FXSAVE gesicherten Register wieder her")
             AddGadgetItem(20, -1, "FXSAVE         speichert die FPU-, MMX-, XMM- und MXCSR-Register in einem 512 Byte grossen Speicherbereich")          
           EndSelect 
         EndIf   
          
           Init1 = 0 : Init2 = 0 : Init3 = 0 : Init4 = 0 : Init5 = 0 : Init6 = 0 : Init7 = 0 : Init8 = 0 : Init9 = 0           
           Panel1Old = Panel1
       EndIf  
;--------------------------------------------------------------------------------------------------
;- Test auf 3DNow! 
       If Panel0 = 1
           Panel2 = GetGadgetState(2)              
           If Init1 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel2Old = 1
             Init1 = 1
             Panel2 = 100
           EndIf 
         If Panel2 <> Panel2Old
           Select Panel2  
;--- Test auf Register-Befehle            
            Case 0 
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "PSWAPW     ldt Operand1 in Operand2 und vertauscht dabei die Words (1-4, 4-1, 2-3, 3-2). Ab Athlon und Duron nicht mehr verfgbar. Ersetzt durch PSWAPD. Keine Untersttzung durch FAsm !")         
;--- Test auf mathematische Befehle 
            Case 1
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "PAVGUSB        berechnet den Durchschnittswert der einzelnen korrespondierenden Bytes und speichert das Ergebnis im ersten Operanden")
             AddGadgetItem(20, -1, "PFACC              addiert die beiden unteren FP-Double-Words des ersten Operanden dort und im oberen Double-Word die Summe der beiden FP-Double-Words des zweiten Operanden")  
             AddGadgetItem(20, -1, "PFADD             addiert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert das Ergebnis im ersten Operanden") 
             AddGadgetItem(20, -1, "PFMUL             multipliziert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert das Ergebnis im ersten Operanden")        
             AddGadgetItem(20, -1, "PFRCP             berechnet den Kehrwert der Fliesskommazahl einfacher Genauigkeit des unteren Double-Words von Operand2 und speichert das Ergebnis in den beiden Double-Words von Op.1")        
             AddGadgetItem(20, -1, "PFRCPIT1       berechnet die erste Iteration des Newton-Raphson-Verfahrens einer vorher mit PFRCP berechneten Zahl")         
             AddGadgetItem(20, -1, "PFRCPIT2       berechnet die zweite Iteration des Newton-Raphson-Verfahrens einer vorher mit PFRCP und PFRCPIT1 berechneten Zahl")           
             AddGadgetItem(20, -1, "PFRSQIT1       berechnet die erste Iteration des Newton-Raphson-Verfahrens einer vorher mit PFSQRT berechneten Zahl")           
             AddGadgetItem(20, -1, "PFRSQRT        berechnet die Quadratwurzel (Fliesskommazahl einfacher Genauigkeit) aus dem unteren D-Word von Op.2 und speichert sie mit 15-Bit-Genauigkeit in beide D-Words von Op.1")           
             AddGadgetItem(20, -1, "PFSUB            subtrahiert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit, und zwar Operand1 = Operand1 - Operand2")           
             AddGadgetItem(20, -1, "PFSUBR         subtrahiert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit, und zwar Operand1 = Operand2 - Operand1")           
             AddGadgetItem(20, -1, "PMULHRW     multipliziert die 4 korrespondierenden vorzeichenbehafteten 16-Bit Integer-Werte und addiert zum unteren Ergebnis-Word 8000h, dann wird das obere Word in Op.1 gespeichert")           
;--- Test auf Konvertierungen       
            Case 2
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PF2ID         konvertiert die beiden Fliesskommazahlen einfacher Genauigkeit des zweiten Operanden in zwei vorzeichenbehaftete 32-Bit-Integer-Zahlen in den ersten Operanden")
             AddGadgetItem(20, -1, "PI2FD         konvertiert die beiden vorzeichenbehafteten 32-Bit-Integer-Zahlen des zweiten Operanden in zwei Fliesskommazahlen einfacher Genauigkeit in den ersten Operanden")
;--- Test auf Vergleiche 
            Case 3
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PFCMPEQ      vergleicht die korrespondierenden Double-Words von Operand1 und Operand2 und setzt bei Gleichheit das entsprechende Double-Word von Operand1 auf 0ffffffffh, sonst auf 0")
             AddGadgetItem(20, -1, "PFCMPGE      vergleicht die korrespondierenden Double-Words von Operand1 und Operand2 und setzt bei grsser/gleich das entsprechende Double-Word von Operand1 auf 0ffffffffh, sonst auf 0")
             AddGadgetItem(20, -1, "PFCMPGT      vergleicht die korrespondierenden Double-Words von Operand1 und Operand2 und setzt bei grsser das entsprechende Double-Word von Operand1 auf 0ffffffffh, sonst auf 0")           
             AddGadgetItem(20, -1, "PFMAX          vergleicht die korrespondierenden Double-Words von Operand1 und Operand2 und schreibt das grssere in das entsprechende Double-Word von Operand1")           
             AddGadgetItem(20, -1, "PFMIN           vergleicht die korrespondierenden Double-Words von Operand1 und Operand2 und schreibt das kleinere in das entsprechende Double-Word von Operand1")           
;--- Test auf Cache-Befehle 
            Case 4
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PREFETCH         ldt mindestens 32 Byte (ab Pentium4: 128 Byte) aus dem Speicher in den Daten-Cache. Sollte verwendet werden wenn sicher ist, dass die Daten sich nicht mehr ndern werden")
             AddGadgetItem(20, -1, "PREFETCHW     ldt mindestens 32 Byte (ab Pentium4: 128 Byte) aus dem Speicher in den Daten-Cache. Sollte verwendet werden wenn die Daten sich noch ndern knnen")
;--- Test auf Sonstiges 
            Case 5
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "FEMMS       definiert den MMX-Status als gelscht (hnlich EMMS bei MMX). Jeder 3DNow!-Code-Block sollte von FEMMS umklammert sein")
           EndSelect
         EndIf               
           
           Init0 = 0 : Init2 = 0 : Init3 = 0 : Init4 = 0 : Init5 = 0 : Init6 = 0 : Init7 = 0 : Init8 = 0 : Init9 = 0           
           Panel2Old = Panel2
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf Extended 3DNow! 
       If Panel0 = 2
           Panel3 = GetGadgetState(3)              
           If Init2 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel3Old = 1
             Init2 = 1
             Panel3 = 100
           EndIf 
         If Panel3 <> Panel3Old
           Select Panel3
;--- Test auf Register-Befehle            
            Case 0 
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "MASKMOVQ           siehe SSE-Register-Befehle")            
             AddGadgetItem(20, -1, "MOVNTQ                siehe SSE-Register-Befehle")              
             AddGadgetItem(20, -1, "PEXTRW                siehe SSE-Register-Befehle")
             AddGadgetItem(20, -1, "PINSRW                 siehe SSE-Register-Befehle")              
             AddGadgetItem(20, -1, "PMOVMSKB           siehe SSE-Register-Befehle")
             AddGadgetItem(20, -1, "PSWAPD                ldt Operand2 in Operand1 und vertauscht dabei unteres und oberes Double-Word. Gibt es seit Athlon und Duron; ersetzte PSWAPW und hat den selben Opcode!")          
;--- Test auf mathematische Befehle 
            Case 1
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "PAVGB                 siehe SSE-Mathematische-Befehle") 
             AddGadgetItem(20, -1, "PAVGW                siehe SSE-Mathematische-Befehle")          
             AddGadgetItem(20, -1, "PFNACC               subtrahiert die zwei Fliesskommazahlen einfacher Genauigkeit jedes Operanden voneinander und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "PFPNACC             subtrahiert die zwei Fliesskommazahlen einfacher Genauigkeit des 1.Operanden voneinander und addiert die zwei Zahlen des 2.Operanden und speichert die Ergebnisse im 1.Op.")  
             AddGadgetItem(20, -1, "PMULHUW          siehe SSE-Mathematische-Befehle")          
             AddGadgetItem(20, -1, "PSADBW             siehe SSE-Mathematische-Befehle")           
;--- TEST auf Konvertierungen       
            Case 2
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PF2IW         konvertiert die beiden SP-FPs des 2.Op. in zwei vorzeichenbehaftete 16-Bit-Integer-Zahlen in den 1.Op. Danach wird unter Beibehaltung des Vorzeichens auf 32-Bit-Integer erweitert")
             AddGadgetItem(20, -1, "PI2FW         konvertiert die beiden vorzeichenbehafteten 16-Bit-Integer-Zahlen des zweiten Operanden in zwei Fliesskommazahlen einfacher Genauigkeit in den ersten Operanden")
;--- Test auf Vergleiche
            Case 3
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PMAXSW              siehe SSE-Vergleiche")           
             AddGadgetItem(20, -1, "PMAXUB               siehe SSE-Vergleiche")           
             AddGadgetItem(20, -1, "PMINSW               siehe SSE-Vergleiche")           
             AddGadgetItem(20, -1, "PMINUB                siehe SSE-Vergleiche")     
;--- Test auf Shuffle-Befehle 
            Case 4
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PSHUFW              siehe SSE-Shuffle-Befehle")
;--- Test auf Cache-Befehle 
            Case 5
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PREFETCHNTA       siehe SSE-Cache-Befehle")
             AddGadgetItem(20, -1, "PREFETCHT0          siehe SSE-Cache-Befehle")
             AddGadgetItem(20, -1, "PREFETCHT1          siehe SSE-Cache-Befehle")         
             AddGadgetItem(20, -1, "PREFETCHT2          siehe SSE-Cache-Befehle")         
             AddGadgetItem(20, -1, "SFENCE                   siehe SSE-Cache-Befehle")      
           EndSelect
         EndIf              
           
           Init0 = 0 : Init1 = 0 : Init3 = 0 : Init4 = 0 : Init5 = 0 : Init6 = 0 : Init7 = 0 : Init8 = 0 : Init9 = 0           
           Panel3Old = Panel3
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf SSE 
       If Panel0 = 3
           Panel4 = GetGadgetState(4)              
           If Init3 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel4Old = 1
             Init3 = 1
             Panel4 = 100
           EndIf 
         If Panel4 <> Panel4Old
           Select Panel4 
;--- Test auf Register-Befehle             
            Case 0
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "LDMXCSR         ldt eine 32-Bit-Speichervariable in das MXCSR-Register (Control-Status-Register). Der Standard-Wert ist 1F80h. Das MXCSR-Register kann mit STMXCSR gesichert werden")
             AddGadgetItem(20, -1, "MASKMOVQ     kopiert die von Operand2 angegebenen Bytes von Operand1 in eine 64-Bit-Speicherstelle")
             AddGadgetItem(20, -1, "MOVAPS           kopiert den zweiten Operanden in den ersten. Bei Verwendung einer Variable muss diese an einer 16-Byte-Adresse ausgerichtet sein")  
             AddGadgetItem(20, -1, "MOVHLPS         kopiert die zwei Fliesskommazahlen einfacher Genauigkeit vom oberen Quad-Word des zweiten Operanden in das untere Quad-Word des ersten Operanden")  
             AddGadgetItem(20, -1, "MOVHPS           kopiert die zwei Fliesskommazahlen einfacher Genauigkeit vom oberen Quad-Word des zweiten Operanden in das obere Quad-Word des ersten Operanden")  
             AddGadgetItem(20, -1, "MOVLHPS         kopiert die zwei Fliesskommazahlen einfacher Genauigkeit vom unteren Quad-Word des zweiten Operanden in das obere Quad-Word des ersten Operanden")  
             AddGadgetItem(20, -1, "MOVLPS           kopiert die zwei Fliesskommazahlen einfacher Genauigkeit vom unteren Quad-Word des zweiten Operanden in das untere Quad-Word des ersten Operanden")  
             AddGadgetItem(20, -1, "MOVMSKPS     kopiert die 4 Vorzeichen-Bits (31, 63, 95 und 127) der vier Fliesskommazahlen einfacher Genauigkeit von Operand2 in Operand1 (Bits 0-3), der ein 32-Bit-Register sein muss")  
             AddGadgetItem(20, -1, "MOVNTPS        kopiert die vier Fliesskommazahlen einfacher Genauigkeit des zweiten Operanden (XMM-Register) in den ersten Operanden (128-Bit-Variable) ohne die Daten zu cachen")  
             AddGadgetItem(20, -1, "MOVNTQ          kopiert den zweiten Operanden (MMX-Register) in den ersten Operanden (64-Bit-Variable) ohne die Daten zu cachen")  
             AddGadgetItem(20, -1, "MOVSS             kopiert eine (die unterste) Fliesskommazahl einfacher Genauigkeit vom zweiten Operanden in den ersten Operanden")  
             AddGadgetItem(20, -1, "MOVUPS           kopiert den zweiten Operanden in den ersten. Bei Verwendung einer Variable muss diese nicht an einer 16-Byte-Adresse ausgerichtet sein")  
             AddGadgetItem(20, -1, "PEXTRW          kopiert das durch Operand3 (8-Bit-Konstante) angegebene Word aus Operand2 (MMX-Register) in das untere Word von Operand1 (32-Bit-Register)")  
             AddGadgetItem(20, -1, "PINSRW           kopiert das durch Operand3 (8-Bit-Konstante) angegebene Word aus Operand2 (32-Bit-Register oder 16-Bit-Variable) in das untere Word von Operand1 (MMX-Register)")  
             AddGadgetItem(20, -1, "PMOVMSKB     kopiert die Vorzeichenbits der Bytes (7, 15, 23 usw.) des zweiten Operanden (MMX- oder XMM-Register) in den ersten Operanden (32-Bit-Register)")  
             AddGadgetItem(20, -1, "STMXCSR        ldt das MXCSR-Register (Control-Status-Register) in eine 32-Bit-Speichervariable. Das MXCSR-Register kann mit LDMXCSR zurckgeschrieben (bzw. beschrieben) werden")  
;--- Test auf Konvertierungen       
            Case 1
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "CVTPI2PS         konvertiert die beiden vorzeichenbehafteten Integer-Double-Words aus dem zweiten Operanden in zwei Fliesskommazahlen einfacher Genauigkeit in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTPS2PI         konvertiert die beiden Fliesskommazahlen einfacher Genauigkeit des zweiten Operanden in zwei vorzeichenbehaftete 32-Bit-Integer-Zahlen (mit Rundung) in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTSI2SS         konvertiert das vorzeichenbehaftete Integer-Double-Word aus dem zweiten Operanden in eine Fliesskommazahl einfacher Genauigkeit in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTSS2SI         konvertiert die Fliesskommazahl einfacher Genauigkeit des zweiten Operanden in eine vorzeichenbehaftete 32-Bit-Integer-Zahl (mit Rundung) in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTTPS2PI       konvertiert die beiden Fliesskommazahlen einfacher Genauigkeit des 2.Operanden in zwei vorzeichenbehaftete 32-Bit-Integer-Zahlen (mit Abschneidung) in den 1.Operanden")
             AddGadgetItem(20, -1, "CVTTSS2SI       konvertiert die Fliesskommazahl einfacher Genauigkeit des zweiten Operanden in eine vorzeichenbehaftete 32-Bit-Integer-Zahl (mit Abschneidung) in den ersten Operanden")
;--- Test auf mathematische Befehle
            Case 2
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "ADDPS              addiert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "ADDSS              addiert die (untersten) Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert das Ergebnis im ersten Operanden")
             AddGadgetItem(20, -1, "DIVPS               dividiert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "DIVSS               dividiert die (untersten) Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert das Ergebnis im ersten Operanden")
             AddGadgetItem(20, -1, "MULPS              multipliziert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "MULSS              multipliziert die (untersten) Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert das Ergebnis im ersten Operanden")
             AddGadgetItem(20, -1, "PAVGB              berechnet den Durchschnittswert der einzelnen vorzeichenlosen korrespondierenden Bytes und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "PAVGW             berechnet den Durchschnittswert der einzelnen vorzeichenlosen korrespondierenden Words und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "PMULHUW       multipliziert jeweils die korrespondierenden vorzeichenlosen Integer-Words der beiden Operanden und speichert die oberen 16-Bit der 32-Bit-Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "PSADBW           berechnet den absoluten Differenzbetrag der korrespond. vorzeichenlosen Bytes und speichert die Gesamtsumme als Word im 1.Op., bei 128-Bit wird eine 2. Summe ermittelt")
             AddGadgetItem(20, -1, "RCPPS              bildet jeweils den Kehrwert aus den vier gepackten Fliesskommazahlen einfacher Genauigkeit des 2.Operanden und speichert die Ergebnisse im 1.Operanden")        
             AddGadgetItem(20, -1, "RCPSS              bildet den Kehrwert aus der (untersten) Fliesskommazahl einfacher Genauigkeit des 2.Operanden und speichert das Ergebnis im 1.Operanden")        
             AddGadgetItem(20, -1, "RSQRTPS         zieht jeweils die Quadrat-Wurzel aus den vier gepackten Fliesskommazahlen einfacher Genauigkeit des 2.Operanden, bildet den Kehrwert und speichert die Ergebnisse im 1.Op.")        
             AddGadgetItem(20, -1, "RSQRTSS         zieht die Quadrat-Wurzel aus der (untersten) Fliesskommazahl einfacher Genauigkeit des 2.Operanden, bildet den Kehrwert und speichert das Ergebnis im 1.Op.")        
             AddGadgetItem(20, -1, "SQRTPS           zieht jeweils die Quadrat-Wurzel aus den vier gepackten Fliesskommazahlen einfacher Genauigkeit des 2.Operanden und speichert die Ergebnisse im 1.Operanden") 
             AddGadgetItem(20, -1, "SQRTSS           zieht die Quadrat-Wurzel aus der (untersten) Fliesskommazahl einfacher Genauigkeit des 2.Operanden und speichert das Ergebnis im 1.Operanden") 
             AddGadgetItem(20, -1, "SUBPS              subtrahiert jeweils die korrespondierenden gepackten Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")        
             AddGadgetItem(20, -1, "SUBSS              subtrahiert die (untersten) Fliesskommazahlen einfacher Genauigkeit der beiden Operanden und speichert das Ergebnis im ersten Operanden")        
;--- Test auf logische Verknpfungen 
            Case 3
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "ANDNPS          der erste Operand wird NOT-verknpft (invertiert), danach erfolgt eine AND-Verknpfung mit dem zweiten Operanden und das Ergebnis wird im ersten Operanden gespeichert")
             AddGadgetItem(20, -1, "ANDPS             AND-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")
             AddGadgetItem(20, -1, "ORPS               OR-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")
             AddGadgetItem(20, -1, "XORPS             XOR-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")
;--- Test auf Vergleiche
            Case 4
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "CMPPS          vergleicht jeweils die korrespondierenden gepackten SP-FPs der beiden ersten Operanden und speichert die Ergebnisse im 1.Operanden. Der 3.Op. gibt die Art des Vergleiches vor")
             AddGadgetItem(20, -1, "CMPSS          vergleicht die beiden (untersten) SP-FPs der beiden ersten Operanden und speichert das Ergebnis im 1.Operanden. Der 3.Operand gibt die Art des Vergleiches vor")
             AddGadgetItem(20, -1, "COMISS         vergleicht die beiden (untersten) SP-FPs der beiden Operanden und setzt entsprechend dem Ergebnis Flags")
             AddGadgetItem(20, -1, "MAXPS          vergleicht jeweils die korrespondierenden gepackten SP-FPs der beiden Operanden und schreibt die grssere in das entsprechende Double-Word von Operand1")
             AddGadgetItem(20, -1, "MAXSS          vergleicht die beiden (untersten) SP-FPs der beiden Operanden und schreibt die grssere in das unterste Double-Word von Operand1")
             AddGadgetItem(20, -1, "MINPS           vergleicht jeweils die korrespondierenden gepackten SP-FPs der beiden Operanden und schreibt die kleinere in das entsprechende Double-Word von Operand1")
             AddGadgetItem(20, -1, "MINSS           vergleicht die beiden (untersten) SP-FPs der beiden Operanden und schreibt die kleinere in das unterste Double-Word von Operand1")
             AddGadgetItem(20, -1, "PMAXSW       vergleicht jeweils die korrespondierenden vorzeichenbehafteten Words der beiden Operanden und schreibt das grssere in das entsprechende Word von Operand1")
             AddGadgetItem(20, -1, "PMAXUB       vergleicht jeweils die korrespondierenden vorzeichenlosen Bytes der beiden Operanden und schreibt das grssere in das entsprechende Byte von Operand1")
             AddGadgetItem(20, -1, "PMINSW       vergleicht jeweils die korrespondierenden vorzeichenbehafteten Words der beiden Operanden und schreibt das kleinere in das entsprechende Word von Operand1")
             AddGadgetItem(20, -1, "PMINUB        vergleicht jeweils die korrespondierenden vorzeichenlosen Bytes der beiden Operanden und schreibt das kleinere in das entsprechende Byte von Operand1")
             AddGadgetItem(20, -1, "UCOMISS      vergleicht die beiden (untersten) SP-FPs der beiden Operanden und setzt entsprechend dem Ergebnis Flags")
;--- Test auf Shuffle-Befehle 
            Case 5
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PSHUFW        kopiert die vier Words des zweiten Operanden in anderer Reihenfolge in den ersten Operanden. Die Reihenfolge bestimmt der dritte Operand")
             AddGadgetItem(20, -1, "SHUFPS         kopiert zwei Double-Words von Operand1 in das untere Quad-Word von Op.1 und 2 Double-Words von Op.2 in das obere Quad-Word von Op.1. Die Reihenfolge bestimmt Op.3")
;--- Test auf Entpack-Befehle 
            Case 6
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "UNPCKHPS    kopiert das 3.und 4.Double-Word von Operand1 in die Stellen 1 und 3 von Operand1 und von Operand2 das 3.und 4. Double-Word an die Stellen 2 und 4 von Operand1")
             AddGadgetItem(20, -1, "UNPCKLPS    kopiert das 2.Double-Word von Operand1 in die Stelle 3 von Operand1 und von Operand2 das 1.und 2. Double-Word an die Stellen 2 und 4 von Operand1. Pos.1 bleibt unverndert")
;--- Test auf Cache-Befehle
            Case 7
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PREFETCHT0        ldt Daten (ab Pentium4: 128 Byte) in den Daten-Cache (in alle Cache-Levels). Sollte fr mehrmalig gebrauchte Daten verwendet werden")
             AddGadgetItem(20, -1, "PREFETCHT1        ldt Daten (ab Pentium4: 128 Byte) in den Daten-Cache (in alle Cache-Levels ausser dem nullten)")
             AddGadgetItem(20, -1, "PREFETCHT2        ldt Daten (ab Pentium4: 128 Byte) in den Daten-Cache (in alle Cache-Levels ausser dem nullten und ersten)")
             AddGadgetItem(20, -1, "PREFETCHNTA     ldt Daten (ab Pentium4: 128 Byte) in den Daten-Cache (in so wenig Cache-Levels wie mglich). Sollte fr einmalig gebrauchte Daten verwendet werden")       
             AddGadgetItem(20, -1, "SFENCE                 fhrt noch eventuell anstehende Schreibzugriffe auf den Speicher aus und schafft so fr nachfolgende Schreibzugriffe klare Verhltnisse (siehe z.B. MOVNTPS)")       
           EndSelect
         EndIf              

           Init0 = 0 : Init1 = 0 : Init2 = 0 : Init4 = 0 : Init5 = 0 : Init6 = 0 : Init7 = 0 : Init8 = 0 : Init9 = 0           
           Panel4Old = Panel4
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf SSE2 
       If Panel0 = 4
           Panel5 = GetGadgetState(5)              
           If Init4 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel5Old = 1
             Init4 = 1
             Panel5 = 100
           EndIf 
         If Panel5 <> Panel5Old
           Select Panel5
;--- Test auf Register-Befehle            
            Case 0 
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "MASKMOVDQU    kopiert die von Operand2 angegebenen Bytes von Operand1 in eine 128-Bit-Speicherstelle")
             AddGadgetItem(20, -1, "MOVAPD              kopiert den zweiten Operanden in den ersten (floating-point). Bei Verwendung einer Variable muss diese an einer 16-Byte-Adresse ausgerichtet sein")  
             AddGadgetItem(20, -1, "MOVDQA              kopiert den zweiten Operanden in den ersten (integer). Bei Verwendung einer Variable muss diese an einer 16-Byte-Adresse ausgerichtet sein")  
             AddGadgetItem(20, -1, "MOVDQU             kopiert den zweiten Operanden in den ersten (integer). Bei Verwendung einer Variable muss diese nicht an einer 16-Byte-Adresse ausgerichtet sein")  
             AddGadgetItem(20, -1, "MOVDQ2Q           kopiert das untere Quad-Word (integer) eines XMM-Registers in ein MMX-Register")  
             AddGadgetItem(20, -1, "MOVHPD             kopiert eine (die oberste) Fliesskommazahl doppelter Genauigkeit vom zweiten Operanden in den ersten Operanden")  
             AddGadgetItem(20, -1, "MOVLPD              kopiert eine (die unterste) Fliesskommazahl doppelter Genauigkeit vom zweiten Operanden in den ersten Operanden")  
             AddGadgetItem(20, -1, "MOVMSKPD        kopiert die 2 Vorzeichen-Bits (63 und 127) der zwei Fliesskommazahlen doppelter Genauigkeit von Operand2 in Operand1 (Bits 0-1), der ein 32-Bit-Register sein muss")  
             AddGadgetItem(20, -1, "MOVNTDQ           kopiert den zweiten Operanden (XMM-Register, integer) in den ersten Operanden (128-Bit-Variable) ohne die Daten zu cachen")  
             AddGadgetItem(20, -1, "MOVNTI               kopiert den zweiten Operanden (32-Bit-Register, integer) in den ersten Operanden (32-Bit-Variable) ohne die Daten zu cachen")  
             AddGadgetItem(20, -1, "MOVNTPD           kopiert den zweiten Operanden (XMM-Register, floating-point) in den ersten Operanden (128-Bit-Variable) ohne die Daten zu cachen")  
             AddGadgetItem(20, -1, "MOVQ2DQ           kopiert ein MMX-Register (integer) in das untere Quad-Word eines XMM-Registers")  
             AddGadgetItem(20, -1, "MOVSD                kopiert eine (die unterste) Fliesskommazahl doppelter Genauigkeit vom zweiten Operanden in den ersten Operanden")  
             AddGadgetItem(20, -1, "MOVUPD             kopiert den zweiten Operanden in den ersten (floating-point). Bei Verwendung einer Variable muss diese nicht an einer 16-Byte-Adresse ausgerichtet sein")  
;--- Test auf Konvertierungen       
            Case 1
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "CVTDQ2PD         konvertiert die zwei vorzeichenbehafteten Integer-Double-Words des zweiten Operanden in zwei Fliesskommazahlen doppelter Genauigkeit in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTDQ2PS         konvertiert die vier vorzeichenbehafteten Integer-Double-Words des zweiten Operanden in vier Fliesskommazahlen einfacher Genauigkeit in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTPD2DQ         konvertiert die zwei Fliesskommazahlen doppelter Genauigkeit des zweiten Operanden in zwei vorzeichenbehaftete Integer-Double-Words in den ersten Operanden (XMM-Reg.)")
             AddGadgetItem(20, -1, "CVTPD2PI           konvertiert die zwei Fliesskommazahlen doppelter Genauigkeit des zweiten Operanden in zwei vorzeichenbehaftete Integer-Double-Words in den ersten Operanden (MMX-Reg.)")
             AddGadgetItem(20, -1, "CVTPD2PS          konvertiert die zwei Fliesskommazahlen doppelter Genauigkeit des zweiten Operanden in zwei Fliesskommazahlen einfacher Genauigkeit in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTPI2PD           konvertiert die zwei vorzeichenbehafteten Integer-Double-Words des zweiten Operanden (64-Bit) in zwei Fliesskommazahlen doppelter Genauigkeit in den ersten Operanden")
             AddGadgetItem(20, -1, "CVTPS2DQ         konvertiert die vier Fliesskommazahlen einfacher Genauigkeit des zweiten Operanden in vier vorzeichenbehaftete Integer-Double-Words in den ersten Operanden")          
             AddGadgetItem(20, -1, "CVTPS2PD         konvertiert die zwei Fliesskommazahlen einfacher Genauigkeit des zweiten Operanden in zwei Fliesskommazahlen doppelter Genauigkeit in den ersten Operanden")          
             AddGadgetItem(20, -1, "CVTSD2SI           konvertiert eine Fliesskommazahl doppelter Genauigkeit des zweiten Operanden in ein vorzeichenbehaftetes Integer-Double-Word in den ersten Operanden (32-Bit-Register)")          
             AddGadgetItem(20, -1, "CVTSD2SS          konvertiert eine Fliesskommazahl doppelter Genauigkeit des zweiten Operanden in eine Fliesskommazahl einfacher Genauigkeit in den ersten Operanden")          
             AddGadgetItem(20, -1, "CVTSI2SD           konvertiert ein vorzeichenbehaftetes Integer-Double-Word des zweiten Operanden (32-Bit) in eine Fliesskommazahl doppelter Genauigkeit in den ersten Operanden")          
             AddGadgetItem(20, -1, "CVTSS2SD          konvertiert eine Fliesskommazahl einfacher Genauigkeit des zweiten Operanden in eine Fliesskommazahl doppelter Genauigkeit in den ersten Operanden")          
             AddGadgetItem(20, -1, "CVTTPD2PI         konvertiert die beiden Fliesskommazahlen doppelter Genauigkeit des 2.Op. in zwei vorzeichenbehaftete Double-Word Integer-Zahlen (mit Abschneidung) in den 1.Op.(MMX-Reg.)")          
             AddGadgetItem(20, -1, "CVTTPD2DQ       konvertiert die beiden Fliesskommazahlen doppelter Genauigkeit des 2.Op. in zwei vorzeichenbehaftete Double-Word Integer-Zahlen (mit Abschneidung) in den 1.Op.(XMM-Reg.)")
             AddGadgetItem(20, -1, "CVTTPS2DQ       konvertiert die vier Fliesskommazahlen einfacher Genauigkeit des 2.Operanden in vier vorzeichenbehaftete Double-Word Integer-Zahlen (mit Abschneidung) in den 1.Operanden")
             AddGadgetItem(20, -1, "CVTTSD2SI        konvertiert eine Fliesskommazahl doppelter Genauigkeit des 2.Operanden in ein vorzeichenbehaftetes Integer-Double-Word (mit Abschneidung) in den 1.Op.(32-Bit-Register)")                                    
;--- Test auf mathematische Befehle
            Case 2
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "ADDPD               addiert jeweils die korrespondierenden gepackten Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "ADDSD               addiert die (untersten) Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert das Ergebnis im unteren Quad-Word des ersten Operanden")
             AddGadgetItem(20, -1, "DIVPD                 dividiert jeweils die korrespondierenden gepackten Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "DIVSD                 dividiert die (untersten) Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert das Ergebnis im unteren Quad-Word des ersten Operanden")
             AddGadgetItem(20, -1, "MULPD               multipliziert jeweils die korrespondierenden gepackten Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "MULSD               multipliziert die (untersten) Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert das Ergebnis im unteren Quad-Word des ersten Operanden")
             AddGadgetItem(20, -1, "PADDQ               addiert die korrespondierenden Quad-Words der beiden Operanden in den ersten Operanden, berlufe finden nur innerhalb der Quad-Words statt, werden aber nicht signalisiert")
             AddGadgetItem(20, -1, "PMULUDQ         multipliziert jeweils ein (64-Bit) bzw. zwei (128-Bit) Double-Word(s) der beiden Operanden miteinander und speichert das Ergebnis jeweils als Quad-Word im ersten Operanden")
             AddGadgetItem(20, -1, "PSUBQ               subtrahiert die korrespondierenden Quad-Words der beiden Operanden in den 1.Operanden, berlufe finden nur innerhalb der Quad-Words statt, werden aber nicht signalisiert")
             AddGadgetItem(20, -1, "SQRTPD            zieht die Quadrat-Wurzel aus den zwei gepackten Fliesskommazahlen doppelter Genauigkeit des zweiten Operanden und speichert die Ergebnisse im ersten Operanden")
             AddGadgetItem(20, -1, "SQRTSD            zieht die Quadrat-Wurzel aus der (untersten) Fliesskommazahl doppelter Genauigkeit des zweiten Operanden und speichert das Ergebnis im ersten Operanden")        
             AddGadgetItem(20, -1, "SUBPD               subtrahiert jeweils die korrespondierenden gepackten Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert die Ergebnisse im ersten Operanden")        
             AddGadgetItem(20, -1, "SUBSD               subtrahiert die (untersten) Fliesskommazahlen doppelter Genauigkeit der beiden Operanden und speichert das Ergebnis im unteren Quad-Word des ersten Operanden")        
;--- Test auf Schiebe-Befehle 
            Case 3
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PSLLDQ         shiftet die Bytes (!) des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach links. Von rechts wird mit Nullen aufgefllt")
             AddGadgetItem(20, -1, "PSRLDQ        shiftet die Bytes (!) des ersten Operanden um die im zweiten Operanden angegebene Anzahl nach rechts. Von links wird mit Nullen aufgefllt")
;--- Test auf logische Verknpfungen
            Case 4 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "ANDNPD          der erste Operand wird NOT-verknpft (invertiert), danach erfolgt eine AND-Verknpfung mit dem zweiten Operanden und das Ergebnis wird im ersten Operanden gespeichert")
             AddGadgetItem(20, -1, "ANDPD             AND-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")
             AddGadgetItem(20, -1, "ORPD               OR-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")
             AddGadgetItem(20, -1, "XORPD             XOR-Verknpfung mit ersten und zweiten Operanden, das Ergebnis steht im ersten Operanden")          
;--- Test auf Vergleiche
            Case 5
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "CMPPD           vergleicht jeweils die korrespondierenden gepackten DP-FPs der beiden ersten Operanden und speichert die Ergebnisse im 1.Operanden. Der 3.Op. gibt die Art des Vergleiches vor")
             AddGadgetItem(20, -1, "CMPSD           vergleicht die beiden (unteren) DP-FPs der beiden ersten Operanden und speichert das Ergebnis im 1.Operanden. Der 3.Operand gibt die Art des Vergleiches vor")
             AddGadgetItem(20, -1, "COMISD          vergleicht die beiden (unteren) DP-FPs der beiden Operanden und setzt entsprechend dem Ergebnis Flags")           
             AddGadgetItem(20, -1, "MAXPD           vergleicht jeweils die korrespondierenden gepackten DP-FPs der beiden Operanden und schreibt die grssere in das entsprechende Quad-Word von Operand1")           
             AddGadgetItem(20, -1, "MAXSD           vergleicht die beiden (unteren) DP-FPs der beiden Operanden und schreibt die grssere in das untere Quad-Word von Operand1")           
             AddGadgetItem(20, -1, "MINPD            vergleicht jeweils die korrespondierenden gepackten DP-FPs der beiden Operanden und schreibt die kleinere in das entsprechende Quad-Word von Operand1")           
             AddGadgetItem(20, -1, "MINSD            vergleicht die beiden (unteren) DP-FPs der beiden Operanden und schreibt die kleinere in das untere Quad-Word von Operand1")           
             AddGadgetItem(20, -1, "UCOMISD       vergleicht die beiden (unteren) DP-FPs der beiden Operanden und setzt entsprechend dem Ergebnis Flags")          
;--- Test auf Shuffle-Befehle 
            Case 6 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PSHUFD         kopiert die vier Double-Words des zweiten Operanden in anderer Reihenfolge in den ersten Operanden. Die Reihenfolge bestimmt der dritte Operand")
             AddGadgetItem(20, -1, "PSHUFHW     kopiert die vier oberen Words des zweiten Operanden in anderer Reihenfolge in das obere Quad-Word des ersten Operanden. Die Reihenfolge bestimmt der dritte Operand")
             AddGadgetItem(20, -1, "PSHUFLW      kopiert die vier unteren Words des zweiten Operanden in anderer Reihenfolge in das untere Quad-Word des ersten Operanden. Die Reihenfolge bestimmt der dritte Operand")           
             AddGadgetItem(20, -1, "SHUFPD         kopiert eines der Quad-Words von Op.1 in das untere Quad-Word von Op.1 und eines der Quad-Words von Op.2 in das obere Quad-Word von Op.1. Die Reihenfolge bestimmt Op.3")           
;--- Test auf Entpack-Befehle
            Case 7
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PUNPCKHQDQ     kopiert das obere Quad-Word von Operand2 in das obere Quad-Word von Operand1 und das obere Quad-Word von Operand1 in das untere Quad-Word von Operand1")
             AddGadgetItem(20, -1, "PUNPCKLQDQ      kopiert das untere Quad-Word von Operand2 in das obere Quad-Word von Operand1. Das untere Quad-Word von Operand1 bleibt unverndert")
             AddGadgetItem(20, -1, "UNPCKHPD          kopiert das obere Quad-Word von Operand1 in das untere Quad-Word von Operand1 und das obere Quad-Word von Operand2 in das obere Quad-Word von Operand1")
             AddGadgetItem(20, -1, "UNPCKLPD           kopiert das untere Quad-Word von Operand2 in das obere Quad-Word von Operand1. Das untere Quad-Word von Operand1 bleibt unverndert")       
;--- Test auf Cache-Befehle
            Case 8 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "CLFLUSH       lscht die Cache-Line, die Operand1 angibt")
             AddGadgetItem(20, -1, "LFENCE         fhrt noch eventuell anstehende Lese-Zugriffe auf den Speicher aus und schafft so fr nachfolgende Lese-Zugriffe klare Verhltnisse")
             AddGadgetItem(20, -1, "MFENCE        fhrt noch eventuell anstehende Schreib-oder Lese-Zugriffe auf den Speicher aus und schafft so fr nachfolgende Schreib-oder Lese-Zugriffe klare Verhltnisse")
;--- Test auf Sonstiges 
            Case 9
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PAUSE           verbessert die Performance von Spin-Wait-Loops (Warteschleifen der CPU). Unterdrckt praktisch vermeintliche Fehler in der Speicherverwaltung")                    
           EndSelect
         EndIf               

           Init0 = 0 : Init1 = 0 : Init2 = 0 : Init3 = 0 : Init5 = 0 : Init6 = 0 : Init7 = 0 : Init8 = 0 : Init9 = 0          
           Panel5Old = Panel5
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf SSE3 
       If Panel0 = 5
           Panel6 = GetGadgetState(6)              
           If Init5 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel6Old = 1
             Init5 = 1
             Panel6 = 100
           EndIf 
         If Panel6 <> Panel6Old
           Select Panel6 
;--- Test auf Register-Befehle           
            Case 0 
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "MOVDDUP           kopiert die (untere) Fliesskommazahl doppelter Genauigkeit von Operand2 in beide Quad-Words von Operand1 (Duplizierung)")
             AddGadgetItem(20, -1, "MOVSHDUP         kopiert zwei Fliesskommazahlen einfacher Genauigkeit von Operand2 in die vier Double-Words von Operand1 (Duplizierung)")            
             AddGadgetItem(20, -1, "MOVSLDUP         kopiert zwei Fliesskommazahlen einfacher Genauigkeit von Operand2 in die vier Double-Words von Operand1 (Duplizierung)")
;--- Test auf Konvertierungen       
            Case 1
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "FISTTP               konvertiert die Fliesskomma-Zahl, die in ST(0) der FPU steht, in eine Integer-Zahl (16-, 32- oder 64-Bit (Speicher-)Variable), wobei immer abgerundet wird")
;--- Test auf mathematische Befehle
            Case 2 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "ADDSUBPD       addiert die oberen Doubles und subtrahiert die unteren (Operand1 minus Operand2)")
             AddGadgetItem(20, -1, "ADDSUBPS       addiert die Singles 2 und 4 und subtrahiert 1 und 3 (Operand1 minus Operand2)") 
             AddGadgetItem(20, -1, "HADDPD            addiert die beiden Doubles von Operand1 in das untere Double-Word von Operand1 und die beiden Doubles von Operand2 in das obere Double-Word von Operand1") 
             AddGadgetItem(20, -1, "HADDPS            addiert jeweils 2 Singles von Operand1 in die beiden unteren Double-Words von Operand1 und jeweils 2 Singles von Operand2 in die beiden oberen Double-Words von Operand1") 
             AddGadgetItem(20, -1, "HSUBPD            subtrahiert die beiden Doubles von Op.1 (D1 minus D2) in das untere D-Word von Op.1 und die beiden Doubles von Op.2 (D1 minus D2) in das obere D-Word von Op.1") 
             AddGadgetItem(20, -1, "HSUBPS            subtrahiert jeweils 2 Singles von Operand1 in die beiden unteren D-Words von Operand1 und jeweils 2 Singles von Operand2 in die beiden oberen D-Words von Operand1") 
;--- Test auf Sonstiges
            Case 3
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "LDDQU              kopiert den zweiten Operanden (Speicherstelle) in den ersten (integer). Die Speicherstelle muss nicht an einer 16-Byte-Adresse ausgerichtet sein")
             AddGadgetItem(20, -1, "MONITOR          fr Hyper-Threading; legt einen bestimmten Speicherbereich fr die berwachung fest. Nur fr Intel-CPUs mit Hyper-Threading!")
             AddGadgetItem(20, -1, "MWAIT               legt einen logischen Prozessor so lange still bis in den Monitor-Bereich geschrieben wird. Nur fr Intel-CPUs mit Hyper-Threading!")
           EndSelect
         EndIf              

           Init0 = 0 : Init1 = 0 : Init2 = 0 : Init3 = 0 : Init4 = 0 : Init6 = 0 : Init7 = 0 : Init8 = 0 : Init9 = 0
           Panel6Old = Panel6
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf SSSE3 
       If Panel0 = 6
           Panel7 = GetGadgetState(7)              
           If Init6 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel7Old = 1
             Init6 = 1
             Panel7 = 100
           EndIf 
         If Panel7 <> Panel7Old
           Select Panel7 
;--- Test auf mathematische Befehle
            Case 0
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PABSB                 ermittelt den Absolut-Wert jedes vorzeichenbehafteten Bytes von Operand2 und schreibt ihn vorzeichenlos in das korrespondierende Byte von Operand1")
             AddGadgetItem(20, -1, "PABSD                 ermittelt den Absolut-Wert jedes vorzeichenbehafteten Double-Words von Operand2 und schreibt ihn vorzeichenlos in das korrespondierende Double-Word von Operand1")
             AddGadgetItem(20, -1, "PABSW                ermittelt den Absolut-Wert jedes vorzeichenbehafteten Words von Operand2 und schreibt ihn vorzeichenlos in das korrespondierende Word von Operand1")            
             AddGadgetItem(20, -1, "PHADDD             die Summen aller paarweisen vorzeichenbehafteten Integer-Double-Words von Operand1 als auch von Operand2 werden vorzeichenbehaftet in Operand1 gespeichert")
             AddGadgetItem(20, -1, "PHADDSW          wie PHADDW, aber die Summen werden saturiert") 
             AddGadgetItem(20, -1, "PHADDW            die Summen aller paarweisen vorzeichenbehafteten Integer-Words von Operand1 als auch von Operand2 werden vorzeichenbehaftet in Operand1 gespeichert") 
             AddGadgetItem(20, -1, "PHSUBD             die Differenzen aller paarweisen vorzeichenbehafteten Integer-Double-Words von Operand1 als auch von Operand2 werden vorzeichenbehaftet in Operand1 gespeichert") 
             AddGadgetItem(20, -1, "PHSUBSW          wie PHSUBW, aber die Differenzen werden saturiert") 
             AddGadgetItem(20, -1, "PHSUBW            die Differenzen aller paarweisen vorzeichenbehafteten Integer-Words von Operand1 als auch von Operand2 werden vorzeichenbehaftet in Operand1 gespeichert") 
             AddGadgetItem(20, -1, "PMADDUBSW    multipliziert die vorzeichenlosen Bytes des 1.Op. mit den vorzeichenbehafteten korrespondierenden Bytes des 2.Op., dann paarweise Addition der Words mit Saturierung")
             AddGadgetItem(20, -1, "PMULHRSW      multipliziert jeweils die korrespondierenden vorzeichenbehafteten Words beiden Operanden, das resultierende D-Word wird beschnitten und im ersten Operanden abgespeichert")
             AddGadgetItem(20, -1, "PSIGNB              ndert die Vorzeichen der Bytes des ersten Operanden in Abhngigkeit von den Vorzeichen der korrespondierenden Bytes des zweiten Operanden bzw. setzt sie auf Null")
             AddGadgetItem(20, -1, "PSIGND              ndert die Vorzeichen der D-Words des ersten Operanden in Abhngigkeit von den Vorzeichen der korrespondierenden D-Words des zweiten Operanden bzw. setzt sie auf Null")
             AddGadgetItem(20, -1, "PSIGNW             ndert die Vorzeichen der Words des ersten Operanden in Abhngigkeit von den Vorzeichen der korrespondierenden Words des zweiten Operanden bzw. setzt sie auf Null")
;--- Test auf Shuffle-Befehle
            Case 1 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PSHUFB             die Reihenfolge der Bytes des ersten Operanden wird gemss der Vorgaben im zweiten Operanden verndert")
;--- Test auf Sonstiges
            Case 2 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PALIGNR            verkettet Operand1 und Operand2 und shiftet das Ganze auf eine durch Operand3 vorgegebene Aligned-Speicher-Adresse")
           EndSelect
         EndIf              

           Init0 = 0 : Init1 = 0 : Init2 = 0 : Init3 = 0 : Init4 = 0 : Init5 = 0 : Init7 = 0 : Init8 = 0 : Init9 = 0
           Panel7Old = Panel7
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf SSE4.1 
       If Panel0 = 7
           Panel8 = GetGadgetState(8)              
           If Init7 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel8Old = 1
             Init7 = 1
             Panel8 = 100
           EndIf 
         If Panel8 <> Panel8Old
           Select Panel8 
;--- Test auf Register-Befehle            
            Case 0 
             ClearGadgetItemList(20)
             AddGadgetItem(20, -1, "BLENDPD             kopiert jeweils die korrespondierenden gepackten DP-FPs von Operand2 in Operand1 gemss der Bit-Maske des 8-bittigen 3.Operanden (Bit0 und Bit1, gesetzt = kopieren)")
             AddGadgetItem(20, -1, "BLENDPS             kopiert jeweils die korrespondierenden gepackten SP-FPs von Operand2 in Operand1 gemss der Bit-Maske des 8-bittigen 3.Operanden (Bit0 bis Bit3, gesetzt = kopieren)")
             AddGadgetItem(20, -1, "BLENDVPD          kopiert jeweils die korrespondierenden gepackten DP-FPs von Operand2 in Operand1 gemss der Bit-Maske des Registers XMM0 (Bit63 und Bit127, gesetzt = kopieren)")
             AddGadgetItem(20, -1, "BLENDVPS          kopiert jeweils die korrespondierenden gepackten SP-FPs von Operand2 in Operand1 gemss der Bit-Maske des Registers XMM0 (Bit31, 63, 95 und 127, gesetzt = kopieren)")             
             AddGadgetItem(20, -1, "EXTRACTPS       kopiert einen SP-FP-Wert von Operand2 (XMM-Reg.) in Operand1 (32-bit Reg. oder Variable). Die Auswahl des SP-FP erfolgt durch die Bits 1 und 2 des 8-bittigen 3.Op.")             
             AddGadgetItem(20, -1, "INSERTPS           kopiert einen SP-FP-Wert von Operand2 (XMM-Reg. oder 32-bit Variable) in Operand1 (XMM-Reg.). Der 8-bittige Operand3 ermglicht div. Optionen")
             AddGadgetItem(20, -1, "MOVNTDQA        kopiert den zweiten Operanden (D-Quad-Variable) in den ersten Operanden (XMM-Register) ohne die Daten zu cachen, wenn Operand2 vom Typ Write Combining ist")             
             AddGadgetItem(20, -1, "PBLENDW          ")            
             AddGadgetItem(20, -1, "PBLENDVB         ")            
             AddGadgetItem(20, -1, "PEXTRB           ")            
             AddGadgetItem(20, -1, "PEXTRD           ")
             ;AddGadgetItem(20, -1, "PEXTRQ           ")   ;nur fr 64-Bit-Betriebssysteme!
             AddGadgetItem(20, -1, "PEXTRW           ")
             AddGadgetItem(20, -1, "PINSRB           ")
             AddGadgetItem(20, -1, "PINSRD           ")
             ;AddGadgetItem(20, -1, "PINSRQ           ")   ;nur fr 64-Bit-Betriebssysteme!
;--- Test auf Konvertierungen       
            Case 1
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PMOVSXBD         ")
             AddGadgetItem(20, -1, "PMOVSXBQ         ")
             AddGadgetItem(20, -1, "PMOVSXBW         ")
             AddGadgetItem(20, -1, "PMOVSXDQ         ")
             AddGadgetItem(20, -1, "PMOVSXWD         ")                                       
             AddGadgetItem(20, -1, "PMOVSXWQ         ")
             AddGadgetItem(20, -1, "PMOVZXBD         ")
             AddGadgetItem(20, -1, "PMOVZXBQ         ")
             AddGadgetItem(20, -1, "PMOVZXBW         ")
             AddGadgetItem(20, -1, "PMOVZXDQ         ")
             AddGadgetItem(20, -1, "PMOVZXWD         ")             
             AddGadgetItem(20, -1, "PMOVZXWQ         ")             
;--- Test auf mathematische Befehle
            Case 2
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "DPPD                    multipliziert jeweils die korrespondierenden gepackten DP-FPs von Op.1 und Op.2 gemss der Bit-Maske des 8-bittigen Op.3 und anschliessend Addition gemss der Bit-Maske")           
             AddGadgetItem(20, -1, "DPPS                    multipliziert jeweils die korrespondierenden gepackten SP-FPs von Op.1 und Op.2 gemss der Bit-Maske des 8-bittigen Op.3 und anschliessend Addition gemss der Bit-Maske")           
             AddGadgetItem(20, -1, "MPSADBW              ")            
             AddGadgetItem(20, -1, "PHMINPOSUW           ")             
             AddGadgetItem(20, -1, "PMAXSB               ")
             AddGadgetItem(20, -1, "PMAXSD               ") 
             AddGadgetItem(20, -1, "PMAXUD               ") 
             AddGadgetItem(20, -1, "PMAXUW               ") 
             AddGadgetItem(20, -1, "PMINSB               ")
             AddGadgetItem(20, -1, "PMINSD               ")             
             AddGadgetItem(20, -1, "PMINUD               ") 
             AddGadgetItem(20, -1, "PMINUW               ")             
             AddGadgetItem(20, -1, "PMULDQ               ")
             AddGadgetItem(20, -1, "PMULLD               ")             
             AddGadgetItem(20, -1, "ROUNDPD              ")              
             AddGadgetItem(20, -1, "ROUNDPS              ")
             AddGadgetItem(20, -1, "ROUNDSD              ")
             AddGadgetItem(20, -1, "ROUNDSS              ")
;--- Test auf Vergleiche
            Case 3
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PCMPEQQ              ")
             AddGadgetItem(20, -1, "PTEST                ")
;--- Test auf Pack-Befehle
            Case 4
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PACKUSDW             ")
           EndSelect
         EndIf              

           Init0 = 0 : Init1 = 0 : Init2 = 0 : Init3 = 0 : Init4 = 0 : Init5 = 0 : Init6 = 0 : Init8 = 0 : Init9 = 0
           Panel8Old = Panel8
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf SSE4.2 
       If Panel0 = 8
           Panel9 = GetGadgetState(9)              
           If Init8 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel9Old = 1
             Init8 = 1
             Panel9 = 100
           EndIf 
         If Panel9 <> Panel9Old
           Select Panel9 
;--- Test auf String-Befehle
            Case 0
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PCMPESTRI            ")
             AddGadgetItem(20, -1, "PCMPESTRM            ") 
             AddGadgetItem(20, -1, "PCMPISTRI            ") 
             AddGadgetItem(20, -1, "PCMPISTRM            ") 
;--- Test auf Vergleiche
            Case 1
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "PCMPGTQ              ")
;--- Test auf Sonstiges
            Case 2 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "CRC32          berechnet den CRC32-Wert von Operand2 (8-, 16-, 32- oder 64-bittig) und schreibt ihn in Operand1 ")            
           EndSelect
         EndIf              

           Init0 = 0 : Init1 = 0 : Init2 = 0 : Init3 = 0 : Init4 = 0 : Init5 = 0 : Init6 = 0 : Init7 = 0 : Init9 = 0
           Panel9Old = Panel9
       EndIf 
;--------------------------------------------------------------------------------------------------
;- Test auf SSE4.2 - POPCNT 
       If Panel0 = 9
           Panel10 = GetGadgetState(10)              
           If Init9 = 0                ;fr erstmaligen Aufruf oder nach Wechsel
             Panel10Old = 1
             Init9 = 1
             Panel10 = 100
           EndIf 
         If Panel10 <> Panel10Old
           Select Panel10 
            Case 0 
             ClearGadgetItemList(20) 
             AddGadgetItem(20, -1, "POPCNT                        ermittelt die Anzahl gesetzter Bits von Operand2 und schreibt sie in Operand1. Die Operanden knnen jeweils 16-, 32- oder 64-bittig sein")
           EndSelect
         EndIf              

           Init0 = 0 : Init1 = 0 : Init2 = 0 : Init3 = 0 : Init4 = 0 : Init5 = 0 : Init6 = 0 : Init7 = 0 : Init8 = 0
           Panel10Old = Panel10
       EndIf
;--------------------------------------------------------------------------------------------------

          Panel20 = GetGadgetState(20) 

          If Panel20 > -1 
            Select Panel0
              Case 0
                Datei = (Panel1 * 100) + Panel20           ;MMX
              Case 1
                Datei = 1000 + (Panel2 * 100) + Panel20    ;3DNow!
              Case 2
                Datei = 2000 + (Panel3 * 100) + Panel20    ;Extended 3DNow!            
              Case 3
                Datei = 3000 + (Panel4 * 100) + Panel20    ;SSE      
              Case 4
                Datei = 4000 + (Panel5 * 100) + Panel20    ;SSE2       
              Case 5
                Datei = 5000 + (Panel6 * 100) + Panel20    ;SSE3       
              Case 6
                Datei = 6000 + (Panel7 * 100) + Panel20    ;SSSE3       
              Case 7
                Datei = 7000 + (Panel8 * 100) + Panel20    ;SSE4.1                   
              Case 8
                Datei = 8000 + (Panel9 * 100) + Panel20    ;SSE4.2 
              Case 9
                Datei = 9000 + (Panel10 * 100) + Panel20   ;SSE4.2 - POPCNT          
            EndSelect

;- Auf Tests, ob die SIMD-Erweiterung fr das aufgerufene Beispiel von der CPU untersttzt wird,
;-  wurde verzichtet, da z.T. mehrere Generationen in einem Beispiel vertreten sind (z.B.MMX-SSE)
;- Bei einer Windows-Fehlermeldung wird mans schon merken ;-)
;- Siehe dazu auch die Kennzeichnungen mit "+" oder "--"             
            If Datei <> DateiOld And Datei < 9999
              DateiOld = Datei  
                Select Datei
                  Case 8000 To 9999
                    MessageRequester("Achtung!", "SSE4.2 wird erst in Intel-CPUs mit Nehalem-Core zur Verfgung stehen !")                  
                  Default 
                    Datei$ = Str(Datei) + ".pb"
                    Prog = RunProgram(Datei$, "", "")
                    If Prog = 0
                      MessageRequester("Achtung!", "Kein Beispiel verfgbar !")
                    EndIf
                EndSelect  
            EndIf 
          EndIf 
   
     Until WaitWindowEvent() = #PB_Event_CloseWindow
  
  EndIf

End 
; IDE Options = PureBasic 4.10 (Windows - x86)
; CursorPosition = 927
; FirstLine = 908
; Folding = -
; EnableAsm
; DisableDebugger